# Generated by Django 4.2.6 on 2024-01-29 09:12

from django.db import migrations, models
import django.db.models.deletion
import wcd_envoyer.utils.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wcd_envoyer', '0002_auto_20230505_0852'),
    ]

    operations = [
        migrations.CreateModel(
            name='EventAvailability',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('event', wcd_envoyer.utils.models.DynamicChoicesField(choices=[], max_length=512, verbose_name='Event')),
                ('available_since', models.TimeField(verbose_name='Available since')),
                ('available_till', models.TimeField(verbose_name='Available till')),
            ],
            options={
                'verbose_name': 'Event availability',
                'verbose_name_plural': 'Event availabilities',
            },
        ),
        migrations.CreateModel(
            name='MessageSchedule',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('send_at', models.DateTimeField(verbose_name='Send at')),
                ('message', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='schedule', to='wcd_envoyer.message', verbose_name='Message')),
            ],
            options={
                'verbose_name': 'Message schedule',
                'verbose_name_plural': 'Message schedules',
            },
        ),
        migrations.CreateModel(
            name='ChannelAvailability',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('available_since', models.TimeField(verbose_name='Available since')),
                ('available_till', models.TimeField(verbose_name='Available till')),
                ('config', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='channel_availability', to='wcd_envoyer.channelconfig', verbose_name='Channel config')),
            ],
            options={
                'verbose_name': 'Channel availability',
                'verbose_name_plural': 'Channel availabilities',
            },
        ),
    ]
