# Generated by Django 4.2.6 on 2024-01-30 07:03

from django.db import migrations, models
import wcd_envoyer.utils.types


class Migration(migrations.Migration):

    dependencies = [
        ('wcd_envoyer', '0002_auto_20230505_0852'),
    ]

    operations = [
        migrations.AlterField(
            model_name='channelconfig',
            name='data',
            field=models.JSONField(blank=True, default=dict, encoder=wcd_envoyer.utils.types.EnvoyerJSONEncoder, verbose_name='Data'),
        ),
        migrations.AlterField(
            model_name='channelconfig',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='message',
            name='data',
            field=models.JSONField(blank=True, default=dict, encoder=wcd_envoyer.utils.types.EnvoyerJSONEncoder, verbose_name='Data'),
        ),
        migrations.AlterField(
            model_name='message',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='message',
            name='recipients',
            field=models.JSONField(default=list, verbose_name='Recipients'),
        ),
        migrations.AlterField(
            model_name='template',
            name='data',
            field=models.JSONField(blank=True, default=dict, encoder=wcd_envoyer.utils.types.EnvoyerJSONEncoder, verbose_name='Data'),
        ),
        migrations.AlterField(
            model_name='template',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='templatetranslation',
            name='data',
            field=models.JSONField(blank=True, default=dict, encoder=wcd_envoyer.utils.types.EnvoyerJSONEncoder, verbose_name='Data'),
        ),
        migrations.AlterField(
            model_name='templatetranslation',
            name='language',
            field=models.CharField(choices=[('ru', 'Russian'), ('uk', 'Ukrainian'), ('pl', 'Polish'), ('cs', 'Czech'), ('ru-md', 'Russian-md'), ('ru-uz', 'Russian-uz'), ('cs-sk', 'Czech-sk'), ('ro', 'Romanian'), ('sk', 'Slovak'), ('en', 'English')], default='ru', max_length=32, verbose_name='Language'),
        ),
    ]
