# Generated by Django 4.2.6 on 2024-01-30 07:03

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wcd_envoyer', '0003_alter_channelconfig_data_alter_channelconfig_id_and_more'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='MessageAttachedUser',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('message', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='attached_users', to='wcd_envoyer.message', verbose_name='Message')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='envoyer_messages', to=settings.AUTH_USER_MODEL, verbose_name='Send at')),
            ],
            options={
                'verbose_name': 'Message schedule',
                'verbose_name_plural': 'Message schedules',
            },
        ),
    ]
